# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
from scipy.spatial.distance import pdist
from scipy.spatial.distance import squareform
import time
def plotFeature(data, labels):
    clusterNum = len(set(labels))
    fig = plt.figure()
    scatterColors = ['black', 'blue', 'green', 'yellow', 'red', 'purple', 'orange', '#BC8F8F', '#8B4513', 'brown']
    ax = fig.add_subplot(111)
    for i in range(0, clusterNum):
        colorSytle = scatterColors[i % len(scatterColors)]
        subCluster = data[np.where(labels == i)]
        ax.scatter(subCluster[:, 0], subCluster[:, 1], c=colorSytle, s=20)
    plt.show()
def MYAffinityPropagation(data,preference,damping,max_iter,convergence_iter,similarity = None,precomputed=False):
    n,m = data.shape
    if precomputed == False:
        similarity = - squareform(pdist(data,metric='euclidean'))
    similarity.flat[::(n+1)] = preference
    stop = np.zeros((n, convergence_iter))
    responsibility = np.zeros((n, n))
    availability = np.zeros((n, n))
    ind = np.arange(n)
    for idx in range(max_iter):
        a_add_s = np.add(availability, similarity)
        maxIdx,maxY = np.argmax(a_add_s, axis=1), np.max(a_add_s, axis=1)
        a_add_s[ind, maxIdx] = - np.inf
        maxSecondY = np.max(a_add_s, axis=1)
        oldResponsibility = similarity - np.tile(maxY.reshape(-1,1), (1,n))
        oldResponsibility[ind, maxIdx] = similarity[ind, maxIdx] - maxSecondY
        responsibility = responsibility*damping + (1-damping)*oldResponsibility
        zero_max_r = np.maximum(responsibility, 0)
        zero_max_r.flat[::n + 1] = responsibility.flat[::n + 1]
        zero_max_r = np.sum(zero_max_r, axis=0)-zero_max_r
        dA = np.diag(zero_max_r).copy()
        oldAvailability = np.minimum(0, zero_max_r)
        oldAvailability.flat[::(n + 1)] = dA
        availability = availability * damping + (1 - damping) * oldAvailability
        E = (np.diag(availability) + np.diag(responsibility)) > 0
        stop[:, idx % convergence_iter] = E
        K = np.sum(E, axis=0)
        if idx >= convergence_iter:
            se = np.sum(stop, axis=1)
            converged = np.sum((se == convergence_iter) + (se == 0)) == n
            if converged and (K > 0):
                break
    exemplars = np.where(np.diag(availability + responsibility) > 0)[0]
    return exemplars,similarity
def extract_cluster(exemplars, similarity):
    labels = np.argmax(similarity[:,exemplars],axis=1)
    for index,exemplar in enumerate(exemplars):
        labels[exemplar]=index
    return labels