# -*- coding: UTF-8 -*-
# 提取停留点
import numpy as np
import matplotlib.pyplot as plt
from scipy.spatial.distance import cdist
import pandas as pd
from scipy.spatial.distance import squareform
import os
import time
def isCorePoint(ptIndex,floorIds,xyArr,timeArr,eps1,eps2,minPts):
    currentFloorId = floorIds[ptIndex]
    xyDisArr = compute_xy_dis(ptIndex,xyArr)
    timeDisArr = compute_time_dis(ptIndex,timeArr)
    flag = (xyDisArr <= eps1) & (timeDisArr <= eps2) & (floorIds == currentFloorId)
    if np.sum(np.where(flag, 1, 0)) >= minPts:
        return True
    return False
def compute_xy_dis(ptIndex,xyArr):
    xyDisArr = cdist(xyArr[ptIndex].reshape(-1,2), xyArr, metric='euclidean')
    return xyDisArr[0]
def compute_time_dis(ptIndex,timeArr):
    timeDisArr = cdist(timeArr[ptIndex].reshape(-1, 1), timeArr.reshape(-1, 1), metric='euclidean')
    return timeDisArr[0]
def updateSeeds(seeds,neighbours,labels,start):
    for neighbour in neighbours:
        if labels[neighbour] == -1 and neighbour >= start:
            seeds.add(neighbour)
    return seeds
def Indoor_STDBSCAN(data,eps1,eps2,minPts):
    n, m = data.shape
    timeArr=data[:,0]
    xyArr = data[:,1:3]
    floorIds = data[:,3]
    labels = np.full((n,), -1)
    clusterId = 0
    start = 0
    for ptIndex in range(n):
        if ptIndex < start:
            continue
        if labels[ptIndex] != -1:
            continue
        if not isCorePoint(ptIndex, floorIds, xyArr, timeArr, eps1, eps2, minPts):
            continue
        labels[ptIndex] = clusterId
        currentFloorId = floorIds[ptIndex]
        neighbours = np.where((compute_xy_dis(ptIndex,xyArr) <= eps1) & (compute_time_dis(ptIndex,timeArr) <= eps2) & (labels == -1) & ((floorIds == currentFloorId)))[0]
        seeds = set()
        seeds = updateSeeds(seeds, neighbours, labels, start)
        while len(seeds) > 0:
            newPoint = seeds.pop()
            labels[newPoint] = clusterId
            queryResults = np.where((compute_xy_dis(newPoint,xyArr) <= eps1) & (compute_time_dis(newPoint,timeArr) <= eps2) & (floorIds == floorIds[newPoint]))[0]
            if len(queryResults) >= minPts:
                seeds = updateSeeds(seeds, queryResults, labels, start)
        clusterId = clusterId + 1
        start = np.argwhere(labels != -1)[-1][0]
    return labels

def deal(root, fileName):
    ds = pd.read_csv(os.path.join(root,fileName), header=None)
    if(len(ds) < 10):
        return
    data = ds[[5, 2, 3, 4]].values
    labels = Indoor_STDBSCAN(data, 2, 60, 10)
    writerFile = open(os.path.join(r"E:\indoorLocationPrediction\6.staypoint_2_60_10_1223",fileName), mode='w')
    ds['lables'] = labels
    for i in set(labels):
        if i == -1:
            continue
        subDs = ds[ds['lables'] == i].values
        start = subDs[0]
        end = subDs[len(subDs) - 1]
        xy = np.mean(subDs[:, 2:4], axis=0)
        writerFile.write('%s,%s,%s,%s,%s,%s,%s,%s,%s\n' % (
        start[0], start[1], end[1], start[5], end[5], end[5] - start[5], xy[0], xy[1], start[4]))
for root, dirs, files in os.walk(r"E:\indoorLocationPrediction\5.personaltimestamp_1223"):
    for fileName in files:
        deal(root,fileName)